@@ -482,9 +482,9 @@ parser_init (struct parser *parser, const struct argp *argp,
     return ENOMEM;
 
   parser->groups = parser->storage;
-  parser->child_inputs = parser->storage + GLEN;
-  parser->long_opts = parser->storage + GLEN + CLEN;
-  parser->short_opts = parser->storage + GLEN + CLEN + LLEN;
+  parser->child_inputs = (void **)((char*) parser->storage + GLEN);
+  parser->long_opts = (struct option *)((char*) parser->storage + GLEN + CLEN);
+  parser->short_opts = (char*) parser->storage + GLEN + CLEN + LLEN;
   parser->opt_data = opt_data;
 
   memset (parser->child_inputs, 0, szs.num_child_inputs * sizeof (void *));
