@@ -501,7 +501,7 @@ void print_for_mkdir (char *, int, mode_t);
 void print_header (struct tar_stat_info *, off_t);
 void read_and (void (*) (void));
 enum read_header read_header (bool);
-enum read_header tar_checksum (union block *header);
+enum read_header tar_checksum (union block *header, bool silent);
 void skip_file (off_t);
 void skip_member (void);
 
