@@ -253,7 +253,9 @@ enum archive_format
   USTAR_FORMAT,                 /* POSIX.1-1988 (ustar) format */
   POSIX_FORMAT,			/* POSIX.1-2001 format */
   STAR_FORMAT,                  /* Star format defined in 1994 */
-  GNU_FORMAT			/* POSIX format with GNU extensions */
+  GNU_FORMAT			/* Same as OLDGNU_FORMAT with one exception:
+                                   see FIXME note for to_chars() function
+                                   (create.c:189) */
 };
 
 /* Information about a sparse file.  */
@@ -278,9 +280,16 @@ struct tar_stat_info
   char          *gname;     /* group name of owner */
   struct stat   stat;       /* regular filesystem stat */
 
+  /* Nanosecond parts of file timestamps (if available) */
+  unsigned long atime_nsec;
+  unsigned long mtime_nsec;
+  unsigned long ctime_nsec;
+  
   off_t archive_file_size;  /* Size of file as stored in the archive.
 			       Equals stat.st_size for non-sparse files */
 
+  bool   is_sparse;         /* Is the file sparse */ 
+  
   size_t sparse_map_avail;  /* Index to the first unused element in
 			       sparse_map array. Zero if the file is
 			       not sparse */
